@@ -16,6 +16,7 @@ import gzip
 import mailbox
 import dateutil
 import datetime
+import traceback
 
 import dateutil.parser as dup
 import httplib as http
@@ -39,10 +40,7 @@ from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 
 from util import iter_in_transaction, pwd, htmlentitydecoder
 
-engine = create_engine('sqlite:///sqllite.db') #, echo=True)
 metadata = MetaData()
-Session = scoped_session(sessionmaker(autoflush=False, bind=engine))
-
 
 stream_classes = dict()
 
@@ -78,23 +76,38 @@ class MetaSqlBase(DeclarativeMeta):
         return staticmethod(clear_inv_keys)
 
 
+class UpdateError(Exception):
+    def __init__(self, reason, ex):
+        self.reason = reason
+        self.ex = ex
+
+
 class SqlBase(object):
     __metaclass__ = MetaSqlBase
     _decl_class_registry = dict()
 
     @classmethod
-    def create_or_update(cls, **remote):
+    def create_or_update(cls, core, **remote):
+        session = core.session()
         try:
-            sql_obj = session.query(cls)\
-                .filter_by(**cls._unique_remote(remote))\
-                .one()
-            if sql_obj.needs_update(remote):
-                sql_obj.update_props(**cls._validate_remote(remote))
+            try:
+                sql_obj = session.query(cls)\
+                    .filter_by(**cls._unique_remote(remote))\
+                    .one()
+                sql_obj.core = core
+                if sql_obj.needs_update(remote):
+                    sql_obj.update_props(**cls._validate_remote(remote))
+                    sql_obj.update(remote)
+            except InvalidRequestError, e:
+                sql_obj = cls(**cls._validate_remote(remote))
+                sql_obj.core = core
+                session.add(sql_obj)
                 sql_obj.update(remote)
-        except InvalidRequestError, e:
-            sql_obj = cls(**cls._validate_remote(remote))
-            session.add(sql_obj)
-            sql_obj.update(remote)
+        except UpdateError, e:
+            print "got exception"
+            print e
+            print "FIXME!"
+#             raise UpdateError('Error', e)
         return sql_obj
 
     def update_props(self, **keys):
@@ -151,14 +164,18 @@ class Stream(Base):
         return join(self.source, self.name)
 
     def update(self, remote):
-        with pwd(CACHE_DIR):
+        session = self.core.session()
+        with pwd(self.core.opts['cache_dir']):
             try:
                 makedirs(self.cache)
             except OSError, e:
                 print e
             remotes = self.remote_iter()
             for remote in iter_in_transaction(session, remotes):
-                self.source_class.create_or_update(**remote)
+#                 try:
+                self.source_class.create_or_update(self.core, **remote)
+#                 except UpdateError, e:
+#                     print 'failed with remote source %s for stream %s' % (remote['identifier'], self.name)
 
 
 class Mailman(Stream):
@@ -191,26 +208,30 @@ class RHMailman(Mailman):
 
     regex = compile(r"""<a href=\"(?P<url>(?P<year>\d{4})-(?P<month>\w*?)/thread.html)\">\[ Thread \]""", re.I | re.S)
     def remote_iter(self):
-        conn = URLGrabber(reget='check_timestamp',
-                          user_agent='jemoeder/2.0',
-                          prefix='http://www.redhat.com',
-                          retry=0)
+        try:
+            conn = self.core.new_grabber(prefix='http://www.redhat.com')
 
-        site = self.archive
-        html = ug.urlopen(site).read()
-        matches = self.regex.finditer(html)
-        match_dicts = (match.groupdict() for match in matches)
+            site = self.archive
+            html = ug.urlopen(site).read()
+            matches = self.regex.finditer(html)
+            match_dicts = (match.groupdict() for match in matches)
 
-        for match_dict in match_dicts:
-            match_dict['cache_url'] = self.url(match_dict['url'])
-            match_dict['cache_file'] = match_dict['url'].replace('/', '-')
-            match_dict['identifier'] = "%s-%s" % ( match_dict['year'], match_dict['month'])
-            match_dict['stream'] = self
-            match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))
+            for match_dict in match_dicts:
+                match_dict['cache_url'] = self.url(match_dict['url'])
+                match_dict['cache_file'] = match_dict['url'].replace('/', '-')
+                match_dict['identifier'] = "%s-%s" % ( match_dict['year'], match_dict['month'])
+                match_dict['stream'] = self
+                match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))
 
-            resp = conn.urlread(up.urlsplit(match_dict['cache_url']).path)
-            match_dict['size'] = len(resp)
-            yield match_dict
+                resp = conn.urlread(up.urlsplit(match_dict['cache_url']).path)
+                match_dict['size'] = len(resp)
+                yield match_dict
+        except KeyError, e:
+            print 'error fetching attribute %s' % e.message
+        except Exception, e:
+            print 'fail doing something big, not sure what\'s going on'
+            traceback.print_exc()
+            raise UpdateError('unknown error', e)
 
     @property
     def source_class(self):
@@ -238,19 +259,26 @@ class MBoxMailman(Mailman):
         return MBoxSource
 
     def remote_iter(self):
-        site = self.archive
-        html = ug.urlopen(site).read()
-        matches = self.regex.finditer(html)
-        match_dicts = (match.groupdict() for match in matches)
-        for match_dict in match_dicts:
-            match_dict['cache_url'] = self.url(match_dict['mbox'])
-            match_dict['cache_file'] = match_dict['mbox']
-            match_dict['identifier'] = match_dict['mbox']
-            match_dict['stream'] = self
-            match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))
-            if type(match_dict['size']) is str:
-                match_dict['size'] = int(match_dict['size'])
-            yield match_dict
+        try:
+            site = self.archive
+            html = ug.urlopen(site).read()
+            matches = self.regex.finditer(html)
+            match_dicts = (match.groupdict() for match in matches)
+            for match_dict in match_dicts:
+                match_dict['cache_url'] = self.url(match_dict['mbox'])
+                match_dict['cache_file'] = match_dict['mbox']
+                match_dict['identifier'] = match_dict['mbox']
+                match_dict['stream'] = self
+                match_dict['month'] = dup.parse('%s %s' % (match_dict['month'], match_dict['year']))
+                if type(match_dict['size']) is str:
+                    match_dict['size'] = int(match_dict['size'])
+                yield match_dict
+        except KeyError, e:
+            print 'error fetching attribute %s' % e.message
+        except Exception, e:
+            print 'fail doing something big, not sure what\'s going on'
+            traceback.print_exc()
+            raise UpdateError('unknown error', e)
 
 
 class FHMailman(MBoxMailman):
@@ -345,12 +373,14 @@ class MBoxSource(Source):
         return MBoxFact
 
     def update(self, remote):
-        self.cache_source(remote['cache_url'])
+        try:
+            self.cache_source(remote['cache_url'])
+            session = self.core.session()
 
-        mbox = self.mbox
-        for email in iter_in_transaction(session, mbox):
-            email = dict(email.items())
-            try:
+            mbox = self.mbox
+            for email in iter_in_transaction(session, mbox):
+                email = dict(email.items())
+                
                 email['message_id'] = email['Message-ID']
                 email['sender'] = email['From']
                 email['sender_domain'] = self.fact.domain_from_sender(email['sender'])
@@ -363,9 +393,14 @@ class MBoxSource(Source):
                     print "TODO: Fix bad date parsing!"
                     email['date'] = datetime.datetime.min
                 email['subject'] = email['Subject']
-                self.fact.create_or_update(**email)
-            except KeyError:
-                continue
+                self.fact.create_or_update(self.core, **email)
+        except KeyError, e:
+            print 'error fetching attribute %s' % e.message
+        except Exception, e:
+            print 'fail doing something big, not sure what\'s going on'
+            traceback.print_exc()
+            raise UpdateError('unknown error', e)
+
 
     #TODO: This needs to be abstracted a bit
     @property
@@ -408,46 +443,49 @@ class RHMailmanSource(Source):
         return RHMailmanFact
 
     def update(self, remote):
-        self.cache_source(remote['cache_url'])
+        try:
+            self.cache_source(remote['cache_url'])
 
-        conn = URLGrabber(reget='check_timestamp',
-                          user_agent='jemoeder/2.0',
-                          prefix='http://www.redhat.com',
-                          retry=0)
+            conn = self.core.new_grabber(prefix='http://www.redhat.com')
 
-        month_page = codecs.open(self.cache_location, encoding='utf_8').read()
-        matches = self.regex.finditer(month_page)
-        emails = (match.groupdict() for match in matches)
-        for email in emails:
-            url = self.stream.archive + self.identifier + '/' + email['message']
-            loc = join(self.stream.cache, self.identifier, email['message'])
+            month_page = codecs.open(self.cache_location, encoding='utf_8').read()
+            matches = self.regex.finditer(month_page)
+            emails = (match.groupdict() for match in matches)
+            for email in emails:
+                url = self.stream.archive + self.identifier + '/' + email['message']
+                loc = join(self.stream.cache, self.identifier, email['message'])
 
-            url_split = up.urlsplit(url)
-            url_path = url_split.path
-            print url_path
-            email_data = conn.urlread(url_path)
-            print len(email_data)
-            with file(loc, 'w') as f:
-                f.write(email_data)
+                url_split = up.urlsplit(url)
+                url_path = url_split.path
+                print url_path
+                email_data = conn.urlread(url_path)
+                print len(email_data)
+                with file(loc, 'w') as f:
+                    f.write(email_data)
 
 #             url_loc = urlretrieve(url, loc)
 #             email_data = htmlentitydecoder(codecs.open(loc, encoding='utf_8').read())
 
-            try:
-                email['sender'] = self.regex_from.search(email_data).group('from')
-            except AttributeError:
-                email['sender'] = 'nobody@example.com'
-            email['date'] = self.regex_date.search(email_data).group('date')
-            email['message_id'] = self.regex_message_id.search(email_data).group('message_id')
-            email['subject'] = self.regex_subject.search(email_data).group('subject')
-            email['source'] = self
-            try:
-                email['date'] = dup.parse(email['date'])
-            except ValueError:
-                print "TODO: Fix bad date parsing!"
-                email['date'] = datetime.datetime.min
-            self.fact.create_or_update(**email)
-
+                try:
+                    email['sender'] = self.regex_from.search(email_data).group('from')
+                except AttributeError:
+                    email['sender'] = 'nobody@example.com'
+                email['date'] = self.regex_date.search(email_data).group('date')
+                email['message_id'] = self.regex_message_id.search(email_data).group('message_id')
+                email['subject'] = self.regex_subject.search(email_data).group('subject')
+                email['source'] = self
+                try:
+                    email['date'] = dup.parse(email['date'])
+                except ValueError:
+                    print "TODO: Fix bad date parsing!"
+                    email['date'] = datetime.datetime.min
+                self.fact.create_or_update(self.core, **email)
+        except KeyError, e:
+            print 'error fetching attribute %s' % e.message
+        except Exception, e:
+            print 'fail doing something big, not sure what\'s going on'
+            traceback.print_exc()
+            raise UpdateError('unknown error', e)
 
 
 class Fact(Base):
@@ -513,6 +551,5 @@ class RHMailmanFact(Fact):
             return cls.domain_regex2.search(sender).group('domain')
 
 
-session = Session()
-
-__all__ = ['Stream', 'FPMailman', 'FHMailman', 'RHMailman', 'Source', 'Fact', 'session', 'Session', 'metadata', 'engine', 'stream_classes']
+__all__ = ['Stream', 'FPMailman', 'FHMailman', 'RHMailman', 'Source',
+           'Fact', 'metadata', 'stream_classes']
